import dbus
from slip.dbus import polkit
from dbus.glib import DBusGMainLoop
import dbus.mainloop.glib
import dbus.service

import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, GObject, GLib

#
# I18N
#
import gettext
DOMAIN = "system-config-kdump"
gettext.textdomain(DOMAIN)
_ = gettext.gettext

# set timeout of dbus calls to 5 minutes
TIMEOUT=300



# all needed for Python-slip, PoilcyKit and dbus
class DBusProxy (GObject.GObject):
    """
    Class used for communication with/via dbus
    """

    def __init__ (self, progress_window):
        GObject.GObject.__init__(self)
        DBusGMainLoop(set_as_default=True)
        try:
            self.bus = dbus.SystemBus()
            self.dbus_object = self.bus.get_object(
                "org.fedoraproject.systemconfig.kdump.mechanism",
                "/org/fedoraproject/systemconfig/kdump/object")
        except dbus.exceptions.DBusException as e:
            print("error %s" %e)

        self.progress_window = progress_window

        # What was on stdout
        self.std = ""

        # What was on stderr
        self.err = ""

        # Which command failed
        self.cmd = ""

        # return code
        self.retcode = 0

        GObject.signal_new("proxy-error", self, GObject.SIGNAL_RUN_FIRST,
            GObject.TYPE_NONE,
            (GObject.TYPE_PYOBJECT,))

        self.loop = GObject.MainLoop()

    def handle_reply(self, cmd_retcode_std_err):
        self.cmd, self.retcode, self.std, self.err = cmd_retcode_std_err
        self.loop.quit()
        self.progress_window.hide()

    def handle_reply_config(self, retcode_err):
        self.retcode, self.err = retcode_err
        self.loop.quit()
        self.progress_window.hide()

    def handle_reply_partitions(self, partitions):
        self.partitions = partitions
        self.loop.quit()
        self.progress_window.hide()

    def handle_error(self, exception):
        self.loop.quit()
        self.progress_window.hide()
        self.emit("proxy-error", exception)
        self.retcode = -1
        self.std = ""
        self.err = ""
        self.cmd = ""

    @polkit.enable_proxy
    def getdefaultkernel (self):
        """
        Return name of default kernel set in bootloader configuration string
        """
        self.progress_window.set_label(_("Getting default kernel name"))
        self.progress_window.show()
        self.dbus_object.getdefaultkernel(
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_reply,
            error_handler = self.handle_error, timeout=TIMEOUT)
        self.loop.run()
        return self.cmd, self.retcode, self.std, self.err

    @polkit.enable_proxy
    def getcmdline (self, kernel):
        """
        Return command line arguments for specific kernel
        """
        self.progress_window.set_label(_("Getting argument for kernel"))
        self.progress_window.show()
        self.dbus_object.getcmdline(
            kernel,
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_reply,
            error_handler = self.handle_error, timeout=TIMEOUT)
        self.loop.run()
        return self.cmd, self.retcode, self.std, self.err

    @polkit.enable_proxy
    def getxencmdline (self, kernel):
        """
        Return command line arguments for specific xen kernel
        """
        self.progress_window.set_label(_("Getting argument for xen kernel"))
        self.progress_window.show()
        self.dbus_object.getxencmdline(
            kernel,
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_reply,
            error_handler = self.handle_error, timeout=TIMEOUT)
        self.loop.run()
        return self.cmd, self.retcode, self.std, self.err

    @polkit.enable_proxy
    def getallkernels (self):
        """
        Return name of all kernels in bootloader configuration files
        separated by newline (\n)
        """
        self.progress_window.set_label(_("Getting name of all kernels"))
        self.progress_window.show()
        self.dbus_object.getallkernels(
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_reply,
            error_handler = self.handle_error, timeout=TIMEOUT)
        self.loop.run()
        return self.cmd, self.retcode, self.std, self.err


    @polkit.enable_proxy
    def writedumpconfig (self, config_string):
        """
        Write config_string string to kdump.conf file and returns error code
        and reread string
        """
        self.progress_window.set_label(_("Writing kdump configuration"))
        self.progress_window.show()
        self.dbus_object.writedumpconfig(
            config_string,
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_reply_config,
            error_handler = self.handle_error, timeout=TIMEOUT)
        self.loop.run()
        return self.retcode, self.err


    @polkit.enable_proxy
    def writebootconfig (self, config_string):
        """
        Update bootloader configuration. config_string is grubby arguments.
        """
        self.progress_window.set_label(_("Writing bootloader configuration"))
        self.progress_window.show()
        self.dbus_object.writebootconfig(
            config_string,
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_reply,
            error_handler = self.handle_error, timeout=TIMEOUT)
        self.loop.run()
        return self.cmd, self.retcode, self.std, self.err


    @polkit.enable_proxy
    def handlekdumpservice(self, chkconfig_status, service_op):
        """
        Start or stop kdump service, turn on or off init script
        """
        self.progress_window.set_label(_("Handling kdump service"))
        self.progress_window.show()
        self.dbus_object.handledumpservice (
            (chkconfig_status, service_op),
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_reply,
            error_handler = self.handle_error, timeout=TIMEOUT)
        self.loop.run()
        return self.cmd, self.retcode, self.std, self.err


    @polkit.enable_proxy
    def getservicestatus(self):
        """
        Get the current status of the kdump service
        """
        self.progress_window.set_label(_("Getting status of kdump service"))
        self.progress_window.show()
        self.dbus_object.getservicestatus(
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_reply,
            error_handler = self.handle_error, timeout=TIMEOUT)
        self.loop.run()
        return self.cmd, self.retcode, self.std, self.err

    @polkit.enable_proxy
    def getunusedpartitions(self):
        """
        Get list of partitions that are not mounted or otherwise used
        """
        self.progress_window.set_label(_("Getting unused partitions"))
        self.progress_window.show()
        self.dbus_object.getunusedpartitions(
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_reply_partitions,
            error_handler = self.handle_error, timeout=TIMEOUT)
        self.loop.run()
        return self.partitions

if __name__ == "__main__":
    dbus_proxy = DBusProxy(None)
